@@ -4,11 +4,13 @@
 #include <stdlib.h>
 #include <sys/types.h>
 
-#include "webgl1.h"
-#include "webgl1_ext.h"
-#include "webgl2.h"
+#include <webgl/webgl1.h>
+#include <webgl/webgl1_ext.h>
+#include <webgl/webgl2.h>
 
-#ifdef __EMSCRIPTEN_PTHREADS__
+#include "webgl_internal.h"
+
+#if defined(__EMSCRIPTEN_PTHREADS__) && defined(__EMSCRIPTEN_OFFSCREEN_FRAMEBUFFER__)
 
 extern EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_do_create_context(const char *target, const EmscriptenWebGLContextAttributes *attributes);
 extern EMSCRIPTEN_RESULT emscripten_webgl_make_context_current_calling_thread(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE context);
@@ -53,7 +55,8 @@ EMSCRIPTEN_WEBGL_CONTEXT_HANDLE emscripten_webgl_create_context(const char *targ
 EMSCRIPTEN_RESULT emscripten_webgl_make_context_current(EMSCRIPTEN_WEBGL_CONTEXT_HANDLE context)
 {
   GL_FUNCTION_TRACE(__func__);
-  if (emscripten_webgl_get_current_context() == context) return;
+  if (emscripten_webgl_get_current_context() == context)
+    return EMSCRIPTEN_RESULT_SUCCESS;
 
   void *owningThread = *(void**)(context + 4);
   if (owningThread == pthread_self())
@@ -126,7 +129,7 @@ void glBufferData(GLenum target, GLsizeiptr size, const void *data, GLenum usage
       void *ptr = memdup(data, size);
       if (ptr || !data) // glBufferData(data=0) can always be handled asynchronously
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glBufferData, ptr, target, size, ptr, usage);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glBufferData, ptr, target, size, ptr, usage);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -148,7 +151,7 @@ void glBufferSubData(GLenum target, GLintptr offset, GLsizeiptr size, const void
       void *ptr = memdup(data, size);
       if (ptr || !data)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glBufferSubData, ptr, target, offset, size, ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glBufferSubData, ptr, target, offset, size, ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -290,7 +293,7 @@ void glTexImage2D(GLenum target, GLint level, GLint internalformat, GLsizei widt
       void *ptr = memdup(pixels, sz);
       if (ptr || !pixels)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexImage2D, ptr, target, level, internalformat, width, height, border, format, type, ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexImage2D, ptr, target, level, internalformat, width, height, border, format, type, ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -318,7 +321,7 @@ void glTexSubImage2D(GLenum target, GLint level, GLint xoffset, GLint yoffset, G
       void *ptr = memdup(pixels, sz);
       if (ptr || !pixels)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexSubImage2D, ptr, target, level, xoffset, yoffset, width, height, format, type, ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIIIIIIII, &emscripten_glTexSubImage2D, ptr, target, level, xoffset, yoffset, width, height, format, type, ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -343,7 +346,7 @@ void glUniform1fv(GLint location, GLsizei count, const GLfloat *value)
       void *ptr = memdup(value, sz);
       if (ptr)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform1fv, ptr, location, count, (GLfloat*)ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform1fv, ptr, location, count, (GLfloat*)ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -368,7 +371,7 @@ void glUniform1iv(GLint location, GLsizei count, const GLint *value)
       void *ptr = memdup(value, sz);
       if (ptr)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform1iv, ptr, location, count, (GLint*)ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform1iv, ptr, location, count, (GLint*)ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -392,7 +395,7 @@ void glUniform2fv(GLint location, GLsizei count, const GLfloat *value)
       void *ptr = memdup(value, sz);
       if (ptr)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform2fv, ptr, location, count, (GLfloat*)ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform2fv, ptr, location, count, (GLfloat*)ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -417,7 +420,7 @@ void glUniform2iv(GLint location, GLsizei count, const GLint *value)
       void *ptr = memdup(value, sz);
       if (ptr)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform2iv, ptr, location, count, (GLint*)ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform2iv, ptr, location, count, (GLint*)ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -441,7 +444,7 @@ void glUniform3fv(GLint location, GLsizei count, const GLfloat *value)
       void *ptr = memdup(value, sz);
       if (ptr)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform3fv, ptr, location, count, (GLfloat*)ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform3fv, ptr, location, count, (GLfloat*)ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -466,7 +469,7 @@ void glUniform3iv(GLint location, GLsizei count, const GLint *value)
       void *ptr = memdup(value, sz);
       if (ptr)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform3iv, ptr, location, count, (GLint*)ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform3iv, ptr, location, count, (GLint*)ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -490,7 +493,7 @@ void glUniform4fv(GLint location, GLsizei count, const GLfloat *value)
       void *ptr = memdup(value, sz);
       if (ptr)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform4fv, ptr, location, count, (GLfloat*)ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform4fv, ptr, location, count, (GLfloat*)ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -515,7 +518,7 @@ void glUniform4iv(GLint location, GLsizei count, const GLint *value)
       void *ptr = memdup(value, sz);
       if (ptr)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform4iv, ptr, location, count, (GLint*)ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIII, &emscripten_glUniform4iv, ptr, location, count, (GLint*)ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -538,7 +541,7 @@ void glUniformMatrix2fv(GLint location, GLsizei count, GLboolean transpose, cons
       void *ptr = memdup(value, sz);
       if (ptr)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix2fv, ptr, location, count, transpose, (GLfloat*)ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix2fv, ptr, location, count, transpose, (GLfloat*)ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -561,7 +564,7 @@ void glUniformMatrix3fv(GLint location, GLsizei count, GLboolean transpose, cons
       void *ptr = memdup(value, sz);
       if (ptr)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix3fv, ptr, location, count, transpose, (GLfloat*)ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix3fv, ptr, location, count, transpose, (GLfloat*)ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -584,7 +587,7 @@ void glUniformMatrix4fv(GLint location, GLsizei count, GLboolean transpose, cons
       void *ptr = memdup(value, sz);
       if (ptr)
       {
-        emscripten_async_queue_on_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix4fv, ptr, location, count, transpose, (GLfloat*)ptr);
+        emscripten_dispatch_to_thread(*(void**)(pthread_getspecific(currentActiveWebGLContext) + 4), EM_FUNC_SIG_VIIII, &emscripten_glUniformMatrix4fv, ptr, location, count, transpose, (GLfloat*)ptr);
         return;
       }
       // Fall through on allocation failure and run synchronously.
@@ -625,7 +628,48 @@ RET_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetQueryObjectuivEXT, GLenum, G
 RET_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetQueryObjecti64vEXT, GLenum, GLenum, GLint64 *);
 RET_SYNC_GL_FUNCTION_3(EM_FUNC_SIG_VIII, void, glGetQueryObjectui64vEXT, GLenum, GLenum, GLuint64 *);
 
-#endif // ~__EMSCRIPTEN_PTHREADS__
+#endif // ~(__EMSCRIPTEN_PTHREADS__ && __EMSCRIPTEN_OFFSCREEN_FRAMEBUFFER__)
+
+// Returns a function pointer to the given WebGL 1 extension function, when queried without
+// a GL extension suffix such as "EXT", "OES", or "ANGLE". This function is used by
+// emscripten_GetProcAddress() to implement legacy GL emulation semantics for portability.
+void *_webgl1_match_ext_proc_address_without_suffix(const char *name)
+{
+  RETURN_FN_WITH_SUFFIX(glGenQueries, EXT);
+  RETURN_FN_WITH_SUFFIX(glDeleteQueries, EXT);
+  RETURN_FN_WITH_SUFFIX(glIsQuery, EXT);
+  RETURN_FN_WITH_SUFFIX(glBeginQuery, EXT);
+  RETURN_FN_WITH_SUFFIX(glEndQuery, EXT);
+  RETURN_FN_WITH_SUFFIX(glQueryCounter, EXT);
+  RETURN_FN_WITH_SUFFIX(glGetQueryiv, EXT);
+  RETURN_FN_WITH_SUFFIX(glGetQueryObjectiv, EXT);
+  RETURN_FN_WITH_SUFFIX(glGetQueryObjectuiv, EXT);
+  RETURN_FN_WITH_SUFFIX(glGetQueryObjecti64v, EXT);
+  RETURN_FN_WITH_SUFFIX(glGetQueryObjectui64v, EXT);
+
+  // WebGL 1 , Extensions
+  RETURN_FN_WITH_SUFFIX(glBindVertexArray, OES);
+  RETURN_FN_WITH_SUFFIX(glDeleteVertexArrays, OES);
+  RETURN_FN_WITH_SUFFIX(glGenVertexArrays, OES);
+  RETURN_FN_WITH_SUFFIX(glIsVertexArray, OES);
+  RETURN_FN_WITH_SUFFIX(glDrawBuffers, WEBGL);
+  RETURN_FN_WITH_SUFFIX(glDrawArraysInstanced, ANGLE);
+  RETURN_FN_WITH_SUFFIX(glDrawElementsInstanced, ANGLE);
+  RETURN_FN_WITH_SUFFIX(glVertexAttribDivisor, ANGLE);
+  RETURN_FN_WITH_SUFFIX(glGenQueries, EXT);
+  RETURN_FN_WITH_SUFFIX(glDeleteQueries, EXT);
+  RETURN_FN_WITH_SUFFIX(glIsQuery, EXT);
+  RETURN_FN_WITH_SUFFIX(glBeginQuery, EXT);
+  RETURN_FN_WITH_SUFFIX(glEndQuery, EXT);
+  RETURN_FN_WITH_SUFFIX(glQueryCounter, EXT);
+  RETURN_FN_WITH_SUFFIX(glGetQueryiv, EXT);
+  RETURN_FN_WITH_SUFFIX(glGetQueryObjectiv, EXT);
+  RETURN_FN_WITH_SUFFIX(glGetQueryObjectuiv, EXT);
+  RETURN_FN_WITH_SUFFIX(glGetQueryObjecti64v, EXT);
+  RETURN_FN_WITH_SUFFIX(glGetQueryObjectui64v, EXT);
+
+  return 0;
+}
 
 void *emscripten_webgl1_get_proc_address(const char *name)
 {
