@@ -2,6 +2,7 @@
 #include <emscripten.h>
 #include <emscripten/threading.h>
 #include <assert.h>
+#include <stdio.h>
 
 void Sleep(double msecs)
 {
@@ -23,10 +24,8 @@ void *thread_main(void *arg)
 	Sleep(1000);
 	Sleep(5000);
 
-#ifdef REPORT_RESULT
-	REPORT_RESULT(1);
-#endif
-	return 0;
+	emscripten_force_exit(0);
+	return NULL;
 }
 
 int main()
@@ -39,5 +38,6 @@ int main()
 	Sleep(5000);
 	pthread_t thread;
 	pthread_create(&thread, NULL, thread_main, NULL);
-	EM_ASM(noExitRuntime=true);
+	emscripten_exit_with_live_runtime();
+	__builtin_unreachable();
 }
